@@ -28,17 +28,12 @@ void ffPrintLogoAndKey(const char* moduleName, uint8_t moduleIndex, const FFModu
                 ffPrintColor(&instance.config.display.colorKeys);
         }
 
-        bool hasIcon = false;
         if (instance.config.display.keyType & FF_MODULE_KEY_TYPE_ICON && moduleArgs && moduleArgs->keyIcon.length > 0)
-        {
             ffStrbufWriteTo(&moduleArgs->keyIcon, stdout);
-            hasIcon = true;
-        }
 
         if (instance.config.display.keyType & FF_MODULE_KEY_TYPE_STRING)
         {
-            if(hasIcon)
-                putchar(' ');
+            ffPrintCharTimes(' ', instance.config.display.keyType >> FF_MODULE_KEY_TYPE_SPACE_SHIFT);
 
             //NULL check is required for modules with custom keys, e.g. disk with the folder path
             if((printType & FF_PRINT_TYPE_NO_CUSTOM_KEY) || !moduleArgs || moduleArgs->key.length == 0)
@@ -100,7 +95,7 @@ void ffPrintFormat(const char* moduleName, uint8_t moduleIndex, const FFModuleAr
     ffStrbufPutTo(&buffer, stdout);
 }
 
-static void printError(const char* moduleName, uint8_t moduleIndex, const FFModuleArgs* moduleArgs, FFPrintType printType, const char* message, va_list arguments)
+void ffPrintError(const char* moduleName, uint8_t moduleIndex, const FFModuleArgs* moduleArgs, FFPrintType printType, const char* message, ...)
 {
     if(!instance.config.display.showErrors)
         return;
@@ -110,7 +105,10 @@ static void printError(const char* moduleName, uint8_t moduleIndex, const FFModu
     if(!instance.config.display.pipe)
         fputs(FASTFETCH_TEXT_MODIFIER_ERROR, stdout);
 
+    va_list arguments;
+    va_start(arguments, message);
     vprintf(message, arguments);
+    va_end(arguments);
 
     if(!instance.config.display.pipe)
         fputs(FASTFETCH_TEXT_MODIFIER_RESET, stdout);
@@ -118,14 +116,6 @@ static void printError(const char* moduleName, uint8_t moduleIndex, const FFModu
     putchar('\n');
 }
 
-void ffPrintError(const char* moduleName, uint8_t moduleIndex, const FFModuleArgs* moduleArgs, FFPrintType printType, const char* message, ...)
-{
-    va_list arguments;
-    va_start(arguments, message);
-    printError(moduleName, moduleIndex, moduleArgs, printType, message, arguments);
-    va_end(arguments);
-}
-
 void ffPrintColor(const FFstrbuf* colorValue)
 {
     //If the color is not set, this would reset in \033[m, which resets everything.
@@ -141,6 +131,12 @@ void ffPrintCharTimes(char c, uint32_t times)
     if(times == 0)
         return;
 
+    if(times == 1)
+    {
+        putchar(c);
+        return;
+    }
+
     char str[32];
     memset(str, c, sizeof(str)); //2 instructions when compiling with AVX2 enabled
     for(uint32_t i = sizeof(str); i <= times; i += (uint32_t)sizeof(str))
